#include "alg/conserved_path.hpp"
#include "mussa_exceptions.hpp"

using namespace std;

/////////////////////

ConservedPath::ConservedPath()
  : window_size(0),
    score(0.0),
    track_indexes()
{
}

ConservedPath::ConservedPath(const ConservedPath& other)
  : window_size(other.window_size),
    score(other.score),
    track_indexes(other.track_indexes)
{
}

ConservedPath::ConservedPath(size_t win_size, double s, ConservedPath::path_type path)
  : window_size(win_size),
    score(s),
    track_indexes(path)
{
}

void ConservedPath::clear()
{
  window_size = 0;
  score = 0.0;
  track_indexes.clear();
}

ConservedPath::path_element&
ConservedPath::operator[](ConservedPath::size_type index)
{
  return track_indexes[index];
}

bool operator==(const ConservedPath& a,
                const ConservedPath& b)
{
  // this isn't good as score is now a double
  return (     (a.score == b.score)
           and (a.track_indexes.size() == b.track_indexes.size())
           and (a.track_indexes == b.track_indexes));
}

bool operator!=(const ConservedPath& a,
                const ConservedPath& b)
{
  return not (a == b);
}

bool operator<(const ConservedPath& a, const ConservedPath &b)
{
  ConservedPath::const_iterator a_i = a.begin();
  ConservedPath::const_iterator b_i = b.begin();
  for(; a_i != a.end() and b_i != b.end(); ++a_i, ++b_i)
  {
    if (*a_i < *b_i) {
      return true;
    } else if (*a_i > *b_i) {
      return false;
    }
    // else they're equal and we continue to loop
  }
  // now handle mismatched lengths
  if (a_i == a.end() and b_i == b.end()) {
    return false; // since the two paths are equal
  } else if (a_i == a.end()) {
    return true; // a is shorter than b
  } else if (b_i == b.end()) {
    return false; // b is shorter than a
  } else {
    // something went horribly wrong
    throw std::runtime_error("ConservedPath operator< had a fatal logic error");
  }
}

std::ostream& operator<<(std::ostream& out, const ConservedPath& path)
{
  out << "<path win_size" << path.window_size 
      << " score=" << path.score 
      << " len=" << path.track_indexes.size();
  if (path.track_indexes.size() > 0)
  {
    // print the first element
    ConservedPath::const_iterator itor = path.begin();
    out << ">" << *itor++;
    // print the remaining elements
    for (;
        itor != path.end();
        ++itor)
    {
      out << ", " << *itor;
    }
    out << "</path>";
  }
  else
  {
    // if we had no elements just close the block
    out << "/>";
  }
  return out;
}

void ConservedPath::push_back(const ConservedPath::path_element& element)
{
  track_indexes.push_back(element);
}

void ConservedPath::pop_back()
{
  track_indexes.pop_back();
}

ConservedPath::size_type ConservedPath::size() const
{
  return track_indexes.size();
}

ConservedPath::iterator ConservedPath::begin()
{
  return track_indexes.begin();
}

ConservedPath::const_iterator ConservedPath::begin() const
{
  return track_indexes.begin();
}

ConservedPath::iterator ConservedPath::end()
{
  return track_indexes.end();
}

ConservedPath::const_iterator ConservedPath::end() const
{
  return track_indexes.end();
}

bool ConservedPath::nextTo(const ConservedPath& next) const
{
  if (size() != next.size() ) {
    throw conserved_path_size_mismatch("paths must be the same length");
  }    
  ConservedPath::const_iterator this_itor = begin();
  ConservedPath::const_iterator next_itor = next.begin();
  for (; this_itor != end(); ++this_itor, ++next_itor)
  {
    if ( (*this_itor + 1) != *next_itor )
      return false;
  }
  return true;
}

vector<bool> ConservedPath::reverseComplimented() const
{
  vector<bool> reversed;
  for (ConservedPath::const_iterator this_itor = begin(); 
       this_itor != end(); 
       ++this_itor)
  {
    if (*this_itor < 0) 
      reversed.push_back(true);
    else
      reversed.push_back(false);
  }
  return reversed;

}

std::vector<ConservedPath::path_element> ConservedPath::normalizedIndexes() const
{
  vector<path_element> paths;
  for (ConservedPath::const_iterator this_itor = begin(); 
       this_itor != end(); 
       ++this_itor)
  {
    if (*this_itor < 0) {
      paths.push_back(-(*this_itor)-window_size);
    } else {
      paths.push_back(*this_itor);
    }
  }
  return paths;
}

ConservedPath& ConservedPath::extend(int growth)
{
  window_size += growth;
  
  // What does this actually do? Tristan's code was doing this operation
  // but I don't understand how it properly adjusts reverse compliment 
  // windows?
  for(ConservedPath::iterator path_i = begin();
      path_i != end();
      ++path_i)
  {
    if (*path_i < 0)
      *path_i += growth;
  }
  return *this;
}

