#include "qui/motif_editor/MotifModel.hpp"

#include <QColor>

const size_t MotifModel::model_column_size = 4;

MotifModel::MotifModel(MussaRef m, QObject *parent) 
  : analysis(m),
    QAbstractTableModel(parent)
{
  const std::set<Sequence>& motif_set = m->motifs();
  boost::shared_ptr<AnnotationColors> cm = m->colorMapper();
  
  std::set<Sequence>::const_iterator motif_i;
  for(motif_i = motif_set.begin();
      motif_i != motif_set.end();
      ++motif_i)
  {
    // ideally we'd know what the motif color was?
    MotifElement new_element(*motif_i, cm->lookup("motif", motif_i->get_sequence()));
    push_back(new_element);
  }
  push_empty();
}

const MussaRef MotifModel::getAnalysis() const {
  return analysis;
}

void MotifModel::assign(
    MotifModel::size_type num, 
    const MotifElement& val
)
{
  motifs.assign(num, val);
}

MotifElement& MotifModel::at(
    MotifModel::size_type index
)
{
  return motifs.at(index);
}

MotifElement& MotifModel::back()
{
  return motifs.back();
}

MotifModel::iterator MotifModel::begin()
{
  return motifs.begin();
}

MotifModel::const_iterator MotifModel::begin() const
{
  return motifs.begin();
}

void MotifModel::clear()
{
  if (motifs.size() != 0) {
    beginRemoveRows(QModelIndex(), 0, motifs.size()-1);
    motifs.clear();
    endRemoveRows();
  }
}

MotifModel::iterator MotifModel::end()
{
  return motifs.end();
}

MotifModel::const_iterator MotifModel::end() const
{
  return motifs.end();
}

bool MotifModel::empty() const
{
  return motifs.empty();
}

MotifElement& MotifModel::operator[](
    MotifModel::size_type index
)
{
  return motifs[index];
}

void MotifModel::pop_back()
{
  int last_element = motifs.size()-1;
  if (last_element >= 0) {
    beginRemoveRows(QModelIndex(), last_element, last_element);
    motifs.pop_back();
    endRemoveRows();
  } 
}

void MotifModel::push_back(const MotifElement& item)
{
  int last_element = motifs.size();
  beginInsertRows(QModelIndex(), last_element, last_element);
  motifs.push_back(item);
  endInsertRows();
}

void MotifModel::push_empty()
{
  MotifElement blank;
  push_back(blank);
}

MotifModel::size_type MotifModel::size() const
{
  return motifs.size();
}

int 
MotifModel::rowCount( const QModelIndex& parent) const
{
  return motifs.size();
}

int 
MotifModel::columnCount(const QModelIndex& parent) const
{
  return model_column_size;
}

QVariant 
MotifModel::data(const QModelIndex &index, int role) const
{
  if (!index.isValid())
    return QVariant();
  
  if (index.row() >= motifs.size())
    return QVariant();

  if (index.column() >= model_column_size)
    return QVariant();

  const MotifElement& motif = motifs[index.row()];
  if (role == Qt::DisplayRole) {
  	switch (index.column() ) {
      case MotifModel::EnabledCell:
        return QVariant(motif.isEnabled());
      break;
      case MotifModel::ColorCell: 
        return QVariant(motif.getQColor());
      break;
  		case MotifModel::NameCell:
        return QString(motif.getName().c_str());
    	break;
      case MotifModel::SequenceCell:
      	return QString(motif.getSequenceText());
    	break;
     }
  }
  return QVariant();
}

QVariant 
MotifModel::headerData(
    int section, 
    Qt::Orientation orientation, 
    int role
) const
{
  if (role != Qt::DisplayRole)
    return QVariant();

  if (orientation == Qt::Horizontal) {
    switch(section) {
      case MotifModel::EnabledCell:
        return QString("Enabled");
        break;
      case MotifModel::ColorCell:
        return QString("Color");
        break;
      case MotifModel::NameCell:
        return QString("Name");
        break;
      case MotifModel::SequenceCell:
        return QString("Sequence");
        break;
      default:
        return QVariant();
        break;
    }
  }
  return QVariant();
}

bool MotifModel::setData(
   const QModelIndex& index, 
   const QVariant &value, 
   int role
)
{
   if (index.isValid() and role == Qt::EditRole) {
    MotifElement& motif = motifs[index.row()];
    switch(index.column()) {
      case MotifModel::EnabledCell:
        motif.setEnabled(value.toBool());
        break;
      case MotifModel::ColorCell:
        motif.setQColor(value.value<QColor>());
        break;
      case MotifModel::NameCell:
        motif.setName(value.toString().toStdString());
        break;
      case MotifModel::SequenceCell:
        motif.setSequence(value.toString().toStdString());
        break;
      default:
        return false;
        break;
    }
    emit dataChanged(index, index);
    // automatically grow the list of motifs when the last one is full
    if  (not back().isEmpty()) {
      push_empty();
    }
    return true;
  } else {
    return false;
  }
}

Qt::ItemFlags MotifModel::flags(const QModelIndex& index) const
{
  if (!index.isValid())
    return Qt::ItemIsEnabled;

  // first column is not editable
  if (index.column() == 0)
    return QAbstractItemModel::flags(index);
  else
    return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
}

